# Copyright 2025-2026 Dorsal Hub LTD
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

MEDIAINFO_DISPLAY_FIELDS = {
    # Track types
    "Audio",
    "Menu",
    "General",
    "Other",
    "Image",
    "Video",
    "Text",
    # Misc
    "url",
    # --- GENERAL / TOP-LEVEL FIELDS ---
    "UniqueID_String",
    "FileSize_String",
    "Duration_String",
    "OverallBitRate_String",
    "FrameRate_String",
    "Encoded_Date",
    "Tagged_Date",
    "Encoded_Application_String",
    "Encoded_Library_String",
    # --- COMMON METADATA ---
    "Title",
    "Album",
    "Performer",
    "Composer",
    "Director",
    "Genre",
    "Comment",
    "Description",
    "Synopsis",
    "Copyright",
    # --- CORE FORMAT & CODEC FIELDS (for all tracks) ---
    "ID",
    "Format_Version",
    "Format_String",
    "Format_Info",
    "Format_Profile",
    "CodecID",
    "CodecID_Info",
    "InternetMediaType",
    "Compression_Mode_String",
    # --- DETAILED ENCODING SETTINGS (primarily for Video) ---
    "Encoded_Library_Settings",
    "Format_Settings",
    "Format_Settings_CABAC_String",
    "Format_Settings_RefFrames_String",
    "Format_Settings_GOP",
    "BitsPixel_Frame",
    # --- VIDEO-SPECIFIC FIELDS ---
    "Width",
    "Height",
    "DisplayAspectRatio_String",
    "FrameRate_Mode_String",
    "ScanType_String",
    "HDR_Format_String",
    # --- AUDIO-SPECIFIC FIELDS ---
    "Channels_String",
    "ChannelLayout",
    "SamplingRate_String",
    # --- IMAGE/COLOR-SPECIFIC FIELDS ---
    "ColorSpace",
    "ChromaSubsampling",
    "BitDepth_String",
    "colour_range",  # Note: UK spelling from MediaInfo
    "colour_primaries",
    "transfer_characteristics",
    "matrix_coefficients",
    "Format_Compression",
    # --- SUBTITLE-SPECIFIC FIELDS ---
    "ElementCount",
    # --- STREAM-LEVEL FIELDS ---
    "BitRate_String",
    "BitRate_Mode_String",
    "StreamSize_String",
    "Language_String",
    "Delay_String",
    "Default_String",
    "Forced_String",
    # --- OTHER / TIMECODE FIELDS ---
    "Type",
    "TimeCode_FirstFrame",
    "TimeCode_LastFrame",
    "TimeCode_Striped_String",
    # --- Fields to Exclude ---
    # --- These are either redundant, too verbose, or not typically shown ---
    # Note: The goal behind selection/exclusion here is to provide similar display fields to the MediaInfo desktop tool
    # "Accompaniment",
    # "ActiveFormatDescription",
    # "Actor",
    # "Actor_Character",
    # "Added_Date",
    # "Album_More",
    # "Album_Performer",
    # "Album_Performer_Sort",
    # "Album_Performer_Url",
    # "Album_ReplayGain_Gain",
    # "Album_ReplayGain_Gain_String",
    # "Album_ReplayGain_Peak",
    # "Album_Sort",
    # "Alignment",
    # "Alignment_String",
    # "AlternateGroup",
    # "AlternateGroup_String",
    # "Archival_Location",
    # "Arranger",
    # "ArtDirector",
    # "AssistantDirector",
    # "Audio_Channels_Total",
    # "Audio_Codec_List",
    # "AudioCount",
    # "Audio_Format_List",
    # "Audio_Format_WithHint_List",
    # "Audio_Language_List",
    # "BarCode",
    # "BitDepth", # Redundant: BitDepth_String is used
    # "BitDepth_Detected",
    # "BitDepth_Detected_String",
    # "BitDepth_Stored",
    # "BitDepth_Stored_String",
    # "BitRate", # Redundant: BitRate_String is used
    # "BitRate_Encoded",
    # "BitRate_Encoded_String",
    # "BitRate_Maximum",
    # "BitRate_Maximum_String",
    # "BitRate_Minimum",
    # "BitRate_Minimum_String",
    # "BitRate_Mode", # Redundant: BitRate_Mode_String is used
    # "BitRate_Nominal",
    # "BitRate_Nominal_String",
    # "Bits-Pixel_Frame", # Alias for BitsPixel_Frame
    # "BPM",
    # "BufferSize",
    # "CatalogNumber",
    # "ChannelLayout_Original",
    # "ChannelLayoutID",
    # "ChannelPositions",
    # "ChannelPositions_Original",
    # "ChannelPositions_Original_String2",
    # "ChannelPositions_String2",
    # "Channels", # Redundant: Channels_String is used
    # "Channels_Original",
    # "Channels_Original_String",
    # "Chapter",
    # "Chapters_Pos_Begin",
    # "Chapters_Pos_End",
    # "Choregrapher",
    # "ChromaSubsampling_Position",
    # "ChromaSubsampling_String", # Redundant: ChromaSubsampling is cleaner
    # "Codec",
    # "Codec_CC",
    # "Codec_Description",
    # "Codec_Extensions",
    # "Codec_Family",
    # "CodecID_Compatible",
    # "CodecID_Description",
    # "CodecID_Hint",
    # "CodecID_String",
    # "CodecID_Url",
    # "CodecID_Version",
    # "Codec_Info", # Redundant: Format_Info is generally better
    # "Codec_Profile",
    # "Codec_Settings", # Too generic, specific settings are used
    # "Codec_Settings_Automatic",
    # "Codec_Settings_BVOP",
    # "Codec_Settings_CABAC", # Redundant: Codec_Settings_CABAC_String is used
    # "Codec_Settings_Endianness",
    # "Codec_Settings_Firm",
    # "Codec_Settings_Floor",
    # "Codec_Settings_GMC",
    # "Codec_Settings_GMC_String",
    # "Codec_Settings_ITU",
    # "Codec_Settings_Law",
    # "Codec_Settings_Matrix",
    # "Codec_Settings_Matrix_Data",
    # "Codec_Settings_PacketBitStream",
    # "Codec_Settings_QPel",
    # "Codec_Settings_RefFrames", # Redundant: Codec_Settings_RefFrames_String is used
    # "Codec_Settings_Sign",
    # "Codec_Url",
    # "CoDirector",
    # "Collection",
    # "Colorimetry",
    # "colour_description_present",
    # "colour_description_present_Original",
    # "colour_description_present_Original_Source",
    # "colour_description_present_Source",
    # "colour_primaries_Original",
    # "colour_primaries_Original_Source",
    # "colour_primaries_Source",
    # "colour_range_Original",
    # "colour_range_Original_Source",
    # "colour_range_Source",
    # "Comic",
    # "Comic_More",
    # "Comic_Position_Total",
    # "CommissionedBy",
    # "Compilation",
    # "Compilation_String",
    # "Composer_Nationality",
    # "Composer_Sort",
    # "Compression_Mode", # Redundant: Compression_Mode_String is used
    # "Compression_Ratio",
    # "Conductor",
    # "ContentType",
    # "CoProducer",
    # "Copyright_Url",
    # "CostumeDesigner",
    # "Count",
    # "Countries",
    # "Country",
    # "Cover",
    # "Cover_Data",
    # "Cover_Description",
    # "Cover_Mime",
    # "Cover_Type",
    # "Cropped",
    # "DataSize",
    # "Default", # Redundant: Default_String is used
    # "Delay", # Redundant: Delay_String is used
    # "Delay_DropFrame",
    # "Delay_Original",
    # "Delay_Original_DropFrame",
    # "Delay_Original_Settings",
    # "Delay_Original_Source",
    # "Delay_Original_String",
    # "Delay_Original_String1",
    # "Delay_Original_String2",
    # "Delay_Original_String3",
    # "Delay_Original_String4",
    # "Delay_Original_String5",
    # "Delay_Settings",
    # "Delay_Source",
    # "Delay_Source_String",
    # "Delay_String1",
    # "Delay_String2",
    # "Delay_String3",
    # "Delay_String4",
    # "Delay_String5",
    # "Dimensions",
    # "DirectorOfPhotography",
    # "Disabled",
    # "Disabled_String",
    # "DisplayAspectRatio", # Redundant: DisplayAspectRatio_String is used
    # "DisplayAspectRatio_CleanAperture",
    # "DisplayAspectRatio_CleanAperture_String",
    # "DisplayAspectRatio_Original",
    # "DisplayAspectRatio_Original_String",
    # "DistributedBy",
    # "DolbyVision_Layers",
    # "DolbyVision_Profile",
    # "DolbyVision_Version",
    # "Domain",
    # "DotsPerInch",
    # "Duration", # Redundant: Duration_String is used
    # "Duration_Base",
    # "Duration_End",
    # "Duration_End_Command",
    # "Duration_End_Command_String",
    # "Duration_End_Command_String1",
    # "Duration_End_Command_String2",
    # "Duration_End_Command_String3",
    # "Duration_End_Command_String4",
    # "Duration_End_Command_String5",
    # "Duration_End_String",
    # "Duration_End_String1",
    # "Duration_End_String2",
    # "Duration_End_String3",
    # "Duration_End_String4",
    # "Duration_End_String5",
    # "Duration_FirstFrame",
    # "Duration_FirstFrame_String",
    # "Duration_FirstFrame_String1",
    # "Duration_FirstFrame_String2",
    # "Duration_FirstFrame_String3",
    # "Duration_FirstFrame_String4",
    # "Duration_FirstFrame_String5",
    # "Duration_LastFrame",
    # "Duration_LastFrame_String",
    # "Duration_LastFrame_String1",
    # "Duration_LastFrame_String2",
    # "Duration_LastFrame_String3",
    # "Duration_LastFrame_String4",
    # "Duration_LastFrame_String5",
    # "Duration_Start",
    # "Duration_Start2End",
    # "Duration_Start2End_String",
    # "Duration_Start2End_String1",
    # "Duration_Start2End_String2",
    # "Duration_Start2End_String3",
    # "Duration_Start2End_String4",
    # "Duration_Start2End_String5",
    # "Duration_Start_Command",
    # "Duration_Start_Command_String",
    # "Duration_Start_Command_String1",
    # "Duration_Start_Command_String2",
    # "Duration_Start_Command_String3",
    # "Duration_Start_Command_String4",
    # "Duration_Start_Command_String5",
    # "Duration_Start_String",
    # "Duration_Start_String1",
    # "Duration_Start_String2",
    # "Duration_Start_String3",
    # "Duration_Start_String4",
    # "Duration_Start_String5",
    # "Duration_String1",
    # "Duration_String2",
    # "Duration_String3",
    # "Duration_String4",
    # "Duration_String5",
    # "EditedBy",
    # "Encoded_Application", # Redundant: Encoded_Application_String is used
    # "Encoded_Application_CompanyName",
    # "Encoded_Application_Name",
    # "Encoded_Application_Url",
    # "Encoded_Application_Version",
    # "EncodedBy",
    # "Encoded_Library", # Redundant: Encoded_Library_String is used
    # "Encoded_Library_CompanyName",
    # "Encoded_Library_Date",
    # "Encoded_Library_Name",
    # "Encoded_Library_Settings", # Redundant: Use specific settings fields
    # "Encoded_Library_Version",
    # "Encoded_OperatingSystem",
    # "Encryption",
    # "Encryption_Format",
    # "Encryption_InitializationVector",
    # "Encryption_Length",
    # "Encryption_Method",
    # "Encryption_Mode",
    # "Encryption_Padding",
    # "EPG_Positions_Begin",
    # "EPG_Positions_End",
    # "Events_MinDuration",
    # "Events_MinDuration_String",
    # "Events_MinDuration_String1",
    # "Events_MinDuration_String2",
    # "Events_MinDuration_String3",
    # "Events_MinDuration_String4",
    # "Events_MinDuration_String5",
    # "Events_PaintOn",
    # "Events_PopOn",
    # "Events_RollUp",
    # "Events_Total",
    # "ExecutiveProducer",
    # "extra",
    # "File_Created_Date",
    # "File_Created_Date_Local",
    # "FileExtension",
    # "FileExtension_Last",
    # "File_Modified_Date",
    # "File_Modified_Date_Local",
    # "FileName",
    # "FileNameExtension",
    # "FileNameExtension_Last",
    # "FileName_Last",
    # "FileSize", # Redundant: FileSize_String is used
    # "FileSize_String1",
    # "FileSize_String2",
    # "FileSize_String3",
    # "FileSize_String4",
    # "FirstDisplay_Delay_Frames",
    # "FirstDisplay_Type",
    # "FirstPacketOrder",
    # "FooterSize",
    # "Forced", # Redundant: Forced_String is used
    # "Format_AdditionalFeatures",
    # "Format_Commercial",
    # "Format_Commercial_IfAny",
    # "Format_Extensions",
    # "Format_Level",
    # "Format_Settings_BVOP",
    # "Format_Settings_BVOP_String",
    # "Format_Settings_Emphasis",
    # "Format_Settings_Endianness",
    # "Format_Settings_Firm",
    # "Format_Settings_Floor",
    # "Format_Settings_FrameMode",
    # "Format_Settings_GMC",
    # "Format_Settings_GMC_String",
    # "Format_Settings_ITU",
    # "Format_Settings_Law",
    # "Format_Settings_Matrix",
    # "Format_Settings_Matrix_Data",
    # "Format_Settings_Matrix_String",
    # "Format_Settings_Mode",
    # "Format_Settings_ModeExtension",
    # "Format_Settings_Packing",
    # "Format_Settings_PictureStructure",
    # "Format_Settings_PS",
    # "Format_Settings_PS_String",
    # "Format_Settings_Pulldown",
    # "Format_Settings_QPel",
    # "Format_Settings_QPel_String",
    # "Format_Settings_SBR",
    # "Format_Settings_SBR_String",
    # "Format_Settings_Sign",
    # "Format_Settings_SliceCount",
    # "Format_Settings_SliceCount_String",
    # "Format_Settings_Wrapping",
    # "Format_Tier",
    # "Format_Url",
    # "FrameCount",
    # "FrameRate", # Redundant: FrameRate_String is used
    # "FrameRate_Den",
    # "FrameRate_Maximum",
    # "FrameRate_Maximum_String",
    # "FrameRate_Minimum",
    # "FrameRate_Minimum_String",
    # "FrameRate_Mode", # Redundant: FrameRate_Mode_String is used
    # "FrameRate_Mode_Original",
    # "FrameRate_Mode_Original_String",
    # "FrameRate_Nominal",
    # "FrameRate_Nominal_String",
    # "FrameRate_Num",
    # "FrameRate_Original",
    # "FrameRate_Original_Den",
    # "FrameRate_Original_Num",
    # "FrameRate_Original_String",
    # "FrameRate_Real",
    # "FrameRate_Real_String",
    # "GeneralCount",
    # "Gop_OpenClosed",
    # "Gop_OpenClosed_FirstFrame",
    # "Gop_OpenClosed_FirstFrame_String",
    # "Gop_OpenClosed_String",
    # "Grouping",
    # "HDR_Format_Commercial",
    # "HDR_Format_Compatibility",
    # "HDR_Format_Level",
    # "HDR_Format_Profile",
    # "HDR_Format_Settings",
    # "HDR_Format_Version",
    # "HeaderSize",
    # "Height_CleanAperture",
    # "Height_CleanAperture_String",
    # "Height_Offset",
    # "Height_Offset_String",
    # "Height_Original",
    # "Height_Original_String",
    # "Height_String", # Redundant: Height (raw value) is used
    # "ICRA",
    # "ID_String", # Redundant: ID is used
    # "Image_Codec_List",
    # "ImageCount",
    # "Image_Format_List",
    # "Image_Format_WithHint_List",
    # "Image_Language_List",
    # "Inform",
    # "Interlacement",
    # "Interlacement_String",
    # "Interleaved",
    # "Interleave_Duration",
    # "Interleave_Duration_String",
    # "Interleave_Preload",
    # "Interleave_Preload_String",
    # "Interleave_VideoFrames",
    # "ISBN",
    # "ISRC",
    # "IsStreamable",
    # "Keywords",
    # "Label",
    # "LabelCode",
    # "Language", # Redundant: Language_String is used
    # "Language_More",
    # "Language_String1",
    # "Language_String2",
    # "Language_String3",
    # "Language_String4",
    # "LawRating",
    # "LawRating_Reason",
    # "LCCN",
    # "Lightness",
    # "Lines_Count",
    # "Lines_MaxCharacterCount",
    # "Lines_MaxCountPerEvent",
    # "List",
    # "List_StreamKind",
    # "List_StreamPos",
    # "List_String",
    # "Lyricist",
    # "Lyrics",
    # "MasteredBy",
    # "Mastered_Date",
    # "MasteringDisplay_ColorPrimaries",
    # "MasteringDisplay_ColorPrimaries_Original",
    # "MasteringDisplay_ColorPrimaries_Original_Source",
    # "MasteringDisplay_ColorPrimaries_Source",
    # "MasteringDisplay_Luminance",
    # "MasteringDisplay_Luminance_Original",
    # "MasteringDisplay_Luminance_Original_Source",
    # "MasteringDisplay_Luminance_Source",
    # "matrix_coefficients_Original",
    # "matrix_coefficients_Original_Source",
    # "matrix_coefficients_Source",
    # "Matrix_ChannelPositions",
    # "Matrix_ChannelPositions_String2",
    # "Matrix_Channels",
    # "Matrix_Channels_String",
    # "Matrix_Format",
    # "MaxCLL",
    # "MaxCLL_Original",
    # "MaxCLL_Original_Source",
    # "MaxCLL_Source",
    # "MaxFALL",
    # "MaxFALL_Original",
    # "MaxFALL_Original_Source",
    # "MaxFALL_Source",
    # "Menu_Codec_List",
    # "MenuCount",
    # "Menu_Format_List",
    # "Menu_Format_WithHint_List",
    # "MenuID",
    # "MenuID_String",
    # "Menu_Language_List",
    # "Mood",
    # "Movie",
    # "Movie_Country",
    # "Movie_More",
    # "Movie_Url",
    # "MultiView_BaseProfile",
    # "MultiView_Count",
    # "MultiView_Layout",
    # "MusicBy",
    # "MuxingMode",
    # "MuxingMode_MoreInfo",
    # "NetworkName",
    # "Original_Album",
    # "Original_Lyricist",
    # "Original_Movie",
    # "Original_NetworkName",
    # "OriginalNetworkName",
    # "Original_Part",
    # "Original_Performer",
    # "Original_Released_Date",
    # "OriginalSourceForm",
    # "OriginalSourceForm_Cropped",
    # "OriginalSourceForm_DistributedBy",
    # "OriginalSourceForm_Name",
    # "OriginalSourceForm_NumColors",
    # "OriginalSourceForm_Sharpness",
    # "OriginalSourceMedium",
    # "OriginalSourceMedium_ID",
    # "OriginalSourceMedium_ID_String",
    # "Original_Track",
    # "Other_Codec_List",
    # "OtherCount",
    # "Other_Format_List",
    # "Other_Format_WithHint_List",
    # "Other_Language_List",
    # "OverallBitRate", # Redundant: OverallBitRate_String is used
    # "OverallBitRate_Maximum",
    # "OverallBitRate_Maximum_String",
    # "OverallBitRate_Minimum",
    # "OverallBitRate_Minimum_String",
    # "OverallBitRate_Mode",
    # "OverallBitRate_Mode_String",
    # "OverallBitRate_Nominal",
    # "OverallBitRate_Nominal_String",
    # "Owner",
    # "PackageName",
    # "Part",
    # "Part_Position",
    # "Part_Position_Total",
    # "Performer_Sort",
    # "Performer_Url",
    # "Period",
    # "PixelAspectRatio",
    # "PixelAspectRatio_CleanAperture",
    # "PixelAspectRatio_CleanAperture_String",
    # "PixelAspectRatio_Original",
    # "PixelAspectRatio_Original_String",
    # "PixelAspectRatio_String",
    # "Played_Count",
    # "Played_First_Date",
    # "Played_Last_Date",
    # "PodcastCategory",
    # "Producer",
    # "Producer_Copyright",
    # "ProductionDesigner",
    # "ProductionStudio",
    # "Publisher",
    # "Publisher_URL",
    # "Rating",
    # "Reel",
    # "Reel_Position",
    # "Reel_Position_Total",
    # "RemixedBy",
    # "ReplayGain_Gain",
    # "ReplayGain_Gain_String",
    # "ReplayGain_Peak",
    # "Resolution",
    # "Resolution_String",
    # "Rotation",
    # "Rotation_String",
    # "Sampled_Height",
    # "Sampled_Width",
    # "SamplesPerFrame",
    # "SamplingCount",
    # "SamplingRate", # Redundant: SamplingRate_String is used
    # "ScanOrder",
    # "ScanOrder_Original",
    # "ScanOrder_Original_String",
    # "ScanOrder_Stored",
    # "ScanOrder_StoredDisplayedInverted",
    # "ScanOrder_Stored_String",
    # "ScanOrder_String",
    # "ScanType", # Redundant: ScanType_String is used
    # "ScanType_Original",
    # "ScanType_Original_String",
    # "ScanType_StoreMethod",
    # "ScanType_StoreMethod_FieldsPerBlock",
    # "ScanType_StoreMethod_String",
    # "ScreenplayBy",
    # "Season",
    # "Season_Position",
    # "Season_Position_Total",
    # "ServiceChannel",
    # "ServiceKind",
    # "ServiceKind_String",
    # "ServiceName",
    # "ServiceProvider",
    # "ServiceProvider_Url",
    # "ServiceType",
    # "Service_Url",
    # "SoundEngineer",
    # "Source_Duration",
    # "Source_Duration_FirstFrame",
    # "Source_Duration_FirstFrame_String",
    # "Source_Duration_FirstFrame_String1",
    # "Source_Duration_FirstFrame_String2",
    # "Source_Duration_FirstFrame_String3",
    # "Source_Duration_FirstFrame_String4",
    # "Source_Duration_FirstFrame_String5",
    # "Source_Duration_LastFrame",
    # "Source_Duration_LastFrame_String",
    # "Source_Duration_LastFrame_String1",
    # "Source_Duration_LastFrame_String2",
    # "Source_Duration_LastFrame_String3",
    # "Source_Duration_LastFrame_String4",
    # "Source_Duration_LastFrame_String5",
    # "Source_Duration_String",
    # "Source_Duration_String1",
    # "Source_Duration_String2",
    # "Source_Duration_String3",
    # "Source_Duration_String4",
    # "Source_Duration_String5",
    # "Source_FrameCount",
    # "Source_SamplingCount",
    # "Source_StreamSize",
    # "Source_StreamSize_Encoded",
    # "Source_StreamSize_Encoded_Proportion",
    # "Source_StreamSize_Encoded_String",
    # "Source_StreamSize_Encoded_String1",
    # "Source_StreamSize_Encoded_String2",
    # "Source_StreamSize_Encoded_String3",
    # "Source_StreamSize_Encoded_String4",
    # "Source_StreamSize_Encoded_String5",
    # "Source_StreamSize_Proportion",
    # "Source_StreamSize_String",
    # "Source_StreamSize_String1",
    # "Source_StreamSize_String2",
    # "Source_StreamSize_String3",
    # "Source_StreamSize_String4",
    # "Source_StreamSize_String5",
    # "Standard",
    # "Status",
    # "Stored_Height",
    # "Stored_Width",
    # "StreamCount",
    # "StreamKind",
    # "StreamKindID",
    # "StreamKindPos",
    # "StreamKind_String",
    # "StreamOrder",
    # "StreamSize", # Redundant: StreamSize_String is used
    # "StreamSize_Demuxed",
    # "StreamSize_Demuxed_String",
    # "StreamSize_Demuxed_String1",
    # "StreamSize_Demuxed_String2",
    # "StreamSize_Demuxed_String3",
    # "StreamSize_Demuxed_String4",
    # "StreamSize_Demuxed_String5",
    # "StreamSize_Encoded",
    # "StreamSize_Encoded_Proportion",
    # "StreamSize_Encoded_String",
    # "StreamSize_Encoded_String1",
    # "StreamSize_Encoded_String2",
    # "StreamSize_Encoded_String3",
    # "StreamSize_Encoded_String4",
    # "StreamSize_Encoded_String5",
    # "StreamSize_Proportion",
    # "StreamSize_String1",
    # "StreamSize_String2",
    # "StreamSize_String3",
    # "StreamSize_String4",
    # "StreamSize_String5",
    # "Subject",
    # "SubTrack",
    # "Summary",
    # "Tagged_Application",
    # "TermsOfUse",
    # "Text_Codec_List",
    # "TextCount",
    # "Text_Format_List",
    # "Text_Format_WithHint_List",
    # "Text_Language_List",
    # "ThanksTo",
    # "TimeCode_DropFrame",
    # "TimeCode_MaxFrameNumber",
    # "TimeCode_MaxFrameNumber_Theory",
    # "TimeCode_Settings",
    # "TimeCode_Source",
    # "TimeCode_Striped", # Redundant: TimeCode_Striped_String is used
    # "TimeCode_Stripped",
    # "TimeCode_Stripped_String",
    # "TimeStamp_FirstFrame",
    # "TimeStamp_FirstFrame_String",
    # "TimeStamp_FirstFrame_String1",
    # "TimeStamp_FirstFrame_String2",
    # "TimeStamp_FirstFrame_String3",
    # "TimeStamp_FirstFrame_String4",
    # "TimeStamp_FirstFrame_String5",
    # "TimeZone",
    # "TimeZones",
    # "Title_More",
    # "Title_Url",
    # "Track",
    # "Track_More",
    # "Track_Position",
    # "Track_Position_Total",
    # "Track_Sort",
    # "Track_Url",
    # "transfer_characteristics_Original",
    # "transfer_characteristics_Original_Source",
    # "transfer_characteristics_Source",
    # "UMID",
    # "UniqueID", # Redundant: UniqueID_String is used
    # "UniversalAdID_Registry",
    # "UniversalAdID_String",
    # "UniversalAdID_Value",
    # "Video0_Delay",
    # "Video0_Delay_String",
    # "Video0_Delay_String1",
    # "Video0_Delay_String2",
    # "Video0_Delay_String3",
    # "Video0_Delay_String4",
    # "Video0_Delay_String5",
    # "Video_Codec_List",
    # "VideoCount",
    # "Video_Delay",
    # "Video_Delay_String",
    # "Video_Delay_String1",
    # "Video_Delay_String2",
    # "Video_Delay_String3",
    # "Video_Delay_String4",
    # "Video_Delay_String5",
    # "Video_Format_List",
    # "Video_Format_WithHint_List",
    # "Video_Language_List",
    # "Width_CleanAperture",
    # "Width_CleanAperture_String",
    # "Width_Offset",
    # "Width_Offset_String",
    # "Width_Original",
    # "Width_Original_String",
    # "Width_String", # Redundant: Width (raw value) is used
    # "WrittenBy",
    # "Written_Location",
}
