{% if panel.data and panel.data.total_sets > 0 %}
<div class="section">
    <h2 class="section-title">{{ panel.title }}</h2>
    <div class="summary-card">
        Found <strong>{{ panel.data.total_sets }}</strong> sets of duplicate files,
        consuming <strong>{{ panel.data.total_wasted_space }}</strong> of wasted space.
    </div>
    <div class="accordion">
    {% for item in panel.data.duplicate_sets %}
        <div class="accordion-item">
            <div class="accordion-header">
                <span>
                    <strong>{{ item.count }} files</strong> of size {{ item.file_size }} each
                    <span class="mono" style="color: var(--muted-text-color); font-size: 0.9em;">(Hash: {{ item.hash }}...)</span>
                </span>
                <span class="accordion-icon">&#9660;</span>
            </div>
            <div class="accordion-content">
                <ul class="file-path-list">
                {% for path in item.paths %}
                    <li class="file-path-item">
                        <span>{{ path | e }}</span>
                        <svg class="copy-icon" title="Copy file path" data-copy-text="{{ path | e }}" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" d="M15.75 17.25v3.375c0 .621-.504 1.125-1.125 1.125h-9.75a1.125 1.125 0 0 1-1.125-1.125V7.875c0-.621.504-1.125 1.125-1.125H6.75a9.06 9.06 0 0 1 1.5.124m7.5 10.376h3.375c.621 0 1.125-.504 1.125-1.125V11.25c0-4.46-3.243-8.161-7.5-8.876a9.06 9.06 0 0 0-1.5-.124H9.375c-.621 0-1.125.504-1.125 1.125v3.5m7.5 10.375H9.375a1.125 1.125 0 0 1-1.125-1.125v-9.25m12 6.625v-1.875a3.375 3.375 0 0 0-3.375-3.375h-1.5a1.125 1.125 0 0 1-1.125-1.125v-1.5a3.375 3.375 0 0 0-3.375-3.375H9.75" /></svg>
                    </li>
                {% endfor %}
                </ul>
            </div>
        </div>
    {% endfor %}
    </div>
</div>
{% endif %}