<div class="section">
    <h2 class="section-title">{{ panel.title }}</h2>
    <div class="file-explorer-controls">
        <input type="text" id="file-explorer-filter" placeholder="Filter files by name...">
    </div>
    <div class="table-container">
        <table id="file-explorer-table">
            <thead>
                <tr>
                    <th data-sort-key="name"><div><span>Name</span></div></th>
                    <th data-sort-key="size"><div><span>Size</span></div></th>
                    <th data-sort-key="media_type"><div><span>Media Type</span></div></th>
                    <th data-sort-key="date_modified"><div><span>Date Modified</span></div></th>
                </tr>
            </thead>
            <tbody>
                </tbody>
        </table>
    </div>
    <div class="pagination-controls" id="file-explorer-pagination">
        </div>
</div>