# Copyright 2025-2026 Dorsal Hub LTD
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

PDFIUM_METADATA_FIELD_MAPPING = {
    "Creator": "creator",
    "Producer": "producer",
    "CreationDate": "creation_date",
    "Author": "author",
    "ModDate": "modified_date",
    "Title": "title",
    "Subject": "subject",
    "Keywords": "keywords",
    "version": "version",
    "page_count": "page_count",
}
